//--------------------------------------------------------------
// File     : stm32_ub_lcd_n95_8gb.c
// Datum    : 13.02.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : GPIO,FSMC
// Funktion : Grafik-LCD (Nokia N95-8GB) (Chip=LDS285)
//            Der Zugriff erfolgt ueber den FSMC-Controller
//            Im 8bit 8080-Mode (8R8G8B = RGB888)
// Hinweis  : Das Display benutzt die CPU-Pins :
//
//      PB0  -> LCD_RESET (16)      PE3  -> LCD_RS  (21)
//      PD0  -> LCD_DB2   (7)       PE7  -> LCD_DB4 (6)
//      PD1  -> LCD_DB3   (18)      PE8  -> LCD_DB5 (19)
//      PD4  -> LCD_RD    (22)      PE9  -> LCD_DB6 (5)
//      PD5  -> LCD_WR    (3)       PE10 -> LCD_DB7 (20)
//      PD7  -> LCD_CS    (4)       
//      PD14 -> LCD_DB0   (8)       VCC (10,12,13,24)
//      PD15 -> LCD_DB1   (17)      GND (1,2,9,11,14,23)
//
// Zahl in Klammer = Pin-Nr vom LCD-Stecker (1...24)   
//          
//--------------------------------------------------------------

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_lcd_n95_8gb.h"



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_N95_InitIO(void);
void P_N95_InitFSMC(void);
void P_N95_InitChip(uint8_t mode);
uint16_t P_N95_ReadID(void);
void P_N95_CMD(uint8_t wert);
void P_N95_DATA(uint8_t wert);
void P_N95_Delay(volatile uint32_t nCount);


//--------------------------------------------------------------
// Init vom LCD-Display
// Return_wert :
//  -> ERROR   , wenn Display nicht gefunden wurde
//  -> SUCCESS , wenn Display OK
//--------------------------------------------------------------
ErrorStatus UB_LCD_Init(void)
{ 
  ErrorStatus ret_wert=ERROR;
  uint16_t lcdid = 0;

  // IO-Lines initialisieren
  P_N95_InitIO(); 
  // FSMC initialisieren
  P_N95_InitFSMC();
  // kleine Pause 
  P_N95_Delay(LCD_N95_PAUSE);
  
  // Reset aktiv
  GPIOB->BSRRH = GPIO_Pin_0;
  // kleine Pause
  P_N95_Delay(LCD_N95_PAUSE);
  // Reset inaktiv
  GPIOB->BSRRL = GPIO_Pin_0;
  // kleine Pause
  P_N95_Delay(LCD_N95_PAUSE);

  // Display-ID auslesen
  // lcdid = P_N95_ReadID(); // buggy
  lcdid=LCD_N95_ID;

  if(lcdid==LCD_N95_ID) {
    // Display gefunden
    ret_wert=SUCCESS;
    // LCD-Controller initialisieren (Mode=Portrait)
    P_N95_InitChip(LCD_N95_PORTRAIT);
    LCD_DISPLAY_MODE=PORTRAIT;
  }  

  LCD_WINDOW.xstart=0;
  LCD_WINDOW.ystart=0;
  LCD_WINDOW.xend=LCD_MAXX-1;
  LCD_WINDOW.yend=LCD_MAXY-1;
  LCD_WINDOW.pixel=LCD_PIXEL;

  return(ret_wert);
}


//--------------------------------------------------------------
// Fllt den Screen mit einer Farbe
//--------------------------------------------------------------
void UB_LCD_FillScreen(uint32_t color)
{
  uint32_t n;
  uint8_t r8,g8,b8;

  r8=(color>>16)&0x0000FF;
  g8=(color>>8)&0x0000FF;
  b8=color&0x0000FF;

  UB_LCD_SetWindow(0,0,LCD_MAXX-1,LCD_MAXY-1);

  P_N95_CMD(LCD_RAMWR);

  for(n=0;n<LCD_PIXEL;n++) {
    P_N95_DATA(r8);
    P_N95_DATA(g8);
    P_N95_DATA(b8);
  }
}


//--------------------------------------------------------------
// Zeichnet ein Pixel mit einer Farbe an x,y Position
//--------------------------------------------------------------
void UB_LCD_DrawPixel(int16_t xpos, int16_t ypos, uint32_t color)
{
  uint8_t r8,g8,b8;

  UB_LCD_SetWindow(xpos,ypos,xpos,ypos);

  r8=(color>>16)&0x0000FF;
  g8=(color>>8)&0x0000FF;
  b8=color&0x0000FF;

  P_N95_CMD(LCD_RAMWR);
  P_N95_DATA(r8);
  P_N95_DATA(g8);
  P_N95_DATA(b8);
}


//--------------------------------------------------------------
// Screen-Mode einstellen
// muss direkt nach dem Init gemacht werden
//
// Mode : [PORTRAIT=default, LANDSCAPE]
//--------------------------------------------------------------
void UB_LCD_SetMode(LCD_MODE_t mode)
{
  if(mode==PORTRAIT) {
    P_N95_InitChip(LCD_N95_PORTRAIT);
    LCD_DISPLAY_MODE=PORTRAIT;
  }
  else {
    P_N95_InitChip(LCD_N95_LANDSCAPE);
    LCD_DISPLAY_MODE=LANDSCAPE;
  }
}


//--------------------------------------------------------------
// stellt ein Display-Fenster zum zeichnen ein
// von xtart,ystart zu xend,yend
//--------------------------------------------------------------
void UB_LCD_SetWindow(uint16_t xstart, uint16_t ystart, uint16_t xend, uint16_t yend)
{
  uint8_t lo,hi;

  P_N95_CMD(LCD_CASET);
  // Horizontal Start
  lo=(xstart&0x00FF);
  hi=(xstart>>8);
  P_N95_DATA(hi);
  P_N95_DATA(lo);
  // Horizontal Ende
  lo=(xend&0x00FF);
  hi=(xend>>8);
  P_N95_DATA(hi);
  P_N95_DATA(lo);

  P_N95_CMD(LCD_RASET);
    // Vertikal Start
  lo=(ystart&0x00FF);
  hi=(ystart>>8);
  P_N95_DATA(hi);
  P_N95_DATA(lo);
  // Vertikal Ende
  lo=(yend&0x00FF);
  hi=(yend>>8);
  P_N95_DATA(hi);
  P_N95_DATA(lo);

  LCD_WINDOW.xstart=xstart;
  LCD_WINDOW.ystart=ystart;
  LCD_WINDOW.xend=xend;
  LCD_WINDOW.yend=yend;
  LCD_WINDOW.pixel=(xend-xstart+1)*(yend-ystart+1);
}


//--------------------------------------------------------------
// interne Funktion
// Init aller IO-Pins fuer das Display
//--------------------------------------------------------------
void P_N95_InitIO(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  //-----------------------------------------
  // Clock Enable von Port-B, Port-D und Port-E
  //-----------------------------------------
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB,ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD,ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE,ENABLE);
  
  //-----------------------------------------
  // Alle Pins von Port-B initialisieren
  //-----------------------------------------
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;              // PB0 -> Reset
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  // Config von Port-B
  GPIO_Init(GPIOB, &GPIO_InitStructure);  
  
  //-----------------------------------------
  // Alle Pins von Port-D initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource0,  GPIO_AF_FSMC); // PD0=FSMC_D2   -> DB2
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource1,  GPIO_AF_FSMC); // PD1=FSMC_D3   -> DB3
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource4,  GPIO_AF_FSMC); // PD4=FSMC_NOE  -> RD
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource5,  GPIO_AF_FSMC); // PD5=FSMC_NWE  -> WR
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource7,  GPIO_AF_FSMC); // PD7=FSMC_NE1  -> CS 
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource14, GPIO_AF_FSMC); // PD14=FSMC_D0  -> DB0
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource15, GPIO_AF_FSMC); // PD15=FSMC_D1  -> DB1
  
  // Structur fuer Port-D
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5 |
                                GPIO_Pin_7 | GPIO_Pin_14 | GPIO_Pin_15;
  
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  // Config von Port-D
  GPIO_Init(GPIOD, &GPIO_InitStructure);   
  
  //-----------------------------------------
  // Alle Pins von Port-E initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource3,  GPIO_AF_FSMC); // PE3=FSMC_A19  -> RS
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource7,  GPIO_AF_FSMC); // PE7=FSMC_D4   -> DB4
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource8,  GPIO_AF_FSMC); // PE8=FSMC_D5   -> DB5  
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource9,  GPIO_AF_FSMC); // PE9=FSMC_D6   -> DB6
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource10, GPIO_AF_FSMC); // PE10=FSMC_D7  -> DB7

  // Structur fuer Port-E
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 |
                                GPIO_Pin_10;
  
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  // Config von Port-E
  GPIO_Init(GPIOE, &GPIO_InitStructure);
}


//--------------------------------------------------------------
// interne Funktion
// Init vom FSMC fuer das Display
// CS an PD7=FSMC_NE1 = Bank-1 / PSRAM-1
//--------------------------------------------------------------
void P_N95_InitFSMC(void)
{
  FSMC_NORSRAMInitTypeDef        FSMC_NORSRAMInitStructure;
  FSMC_NORSRAMTimingInitTypeDef  FSMC_NORSRAMTimingInitStructure;
  
  //-----------------------------------------
  // Clock Enable von FSMC
  //-----------------------------------------
  RCC_AHB3PeriphClockCmd(RCC_AHB3Periph_FSMC, ENABLE);  
  
  //-----------------------------------------
  // Structure fuer das Timing
  //-----------------------------------------  
  FSMC_NORSRAMTimingInitStructure.FSMC_AddressSetupTime = LCD_N95_FSMC_AST;
  FSMC_NORSRAMTimingInitStructure.FSMC_AddressHoldTime = 1;
  FSMC_NORSRAMTimingInitStructure.FSMC_DataSetupTime = LCD_N95_FSMC_DST;
  FSMC_NORSRAMTimingInitStructure.FSMC_BusTurnAroundDuration = 0;
  FSMC_NORSRAMTimingInitStructure.FSMC_CLKDivision = 0;
  FSMC_NORSRAMTimingInitStructure.FSMC_DataLatency = 0;
  FSMC_NORSRAMTimingInitStructure.FSMC_AccessMode = FSMC_AccessMode_A;

  //-----------------------------------------
  // Structure fuer Bank-1 / PSRAM-1
  //-----------------------------------------  
  FSMC_NORSRAMInitStructure.FSMC_Bank = FSMC_Bank1_NORSRAM1;
  FSMC_NORSRAMInitStructure.FSMC_DataAddressMux = FSMC_DataAddressMux_Disable;
  FSMC_NORSRAMInitStructure.FSMC_MemoryType = FSMC_MemoryType_SRAM;
  FSMC_NORSRAMInitStructure.FSMC_MemoryDataWidth = FSMC_MemoryDataWidth_16b; // muss 16bit !!
  FSMC_NORSRAMInitStructure.FSMC_BurstAccessMode = FSMC_BurstAccessMode_Disable;
  FSMC_NORSRAMInitStructure.FSMC_AsynchronousWait = FSMC_AsynchronousWait_Disable;   
  FSMC_NORSRAMInitStructure.FSMC_WaitSignalPolarity = FSMC_WaitSignalPolarity_Low;  
  FSMC_NORSRAMInitStructure.FSMC_WrapMode = FSMC_WrapMode_Disable;  
  FSMC_NORSRAMInitStructure.FSMC_WaitSignalActive = FSMC_WaitSignalActive_BeforeWaitState;  
  FSMC_NORSRAMInitStructure.FSMC_WriteOperation = FSMC_WriteOperation_Enable;  
  FSMC_NORSRAMInitStructure.FSMC_WaitSignal = FSMC_WaitSignal_Disable;   
  FSMC_NORSRAMInitStructure.FSMC_ExtendedMode = FSMC_ExtendedMode_Disable;  
  FSMC_NORSRAMInitStructure.FSMC_WriteBurst = FSMC_WriteBurst_Disable;  
  FSMC_NORSRAMInitStructure.FSMC_ReadWriteTimingStruct = &FSMC_NORSRAMTimingInitStructure; 
  FSMC_NORSRAMInitStructure.FSMC_WriteTimingStruct = &FSMC_NORSRAMTimingInitStructure;
  
  // Config vom FSMC
  FSMC_NORSRAMInit(&FSMC_NORSRAMInitStructure); 
  
  // Enable Bank-1 / PSRAM-1
  FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM1, ENABLE);
}

//--------------------------------------------------------------
// interne Funktion
// initialisiert den LDS285-Controller im Display
// mode :  0x60=Portrait, 0x40=Landscape
//--------------------------------------------------------------
void P_N95_InitChip(uint8_t mode)
{

  P_N95_CMD(LCD_SLPOUT);

  // kleine pause
  P_N95_Delay(LCD_N95_PAUSE);
  
  P_N95_CMD(LCD_INVOFF);
  P_N95_CMD(LCD_IDMOFF);
  P_N95_CMD(LCD_NORON);

  P_N95_CMD(LCD_MADCTR);
  P_N95_DATA(mode);

  P_N95_CMD(LCD_COLMOD);
  P_N95_DATA(0x77);

  P_N95_CMD(LCD_WRDISBV);
  P_N95_DATA(0xFF);
 
  P_N95_CMD(LCD_WRCTRLD);
  P_N95_DATA(0x24);

  P_N95_CMD(LCD_WRCABC);
  P_N95_DATA(0x01);

  P_N95_CMD(LCD_CASET);
  P_N95_DATA(0x00);  // Horizontal Start
  P_N95_DATA(0x00);
  P_N95_DATA(0x00);  // Horizontal Ende
  P_N95_DATA(0xEF);

  P_N95_CMD(LCD_RASET);
  P_N95_DATA(0x00);  // Vertikal Start
  P_N95_DATA(0x00);
  P_N95_DATA(0x01);  // Vertikal Ende
  P_N95_DATA(0x3F);

  // kleine pause
  P_N95_Delay(LCD_N95_PAUSE);

  P_N95_CMD(LCD_DISPON);
}


//--------------------------------------------------------------
// interne Funktion
// Kommando an Display senden
//--------------------------------------------------------------
void P_N95_CMD(uint8_t wert)
{
  LCD_REG = wert;
}


//--------------------------------------------------------------
// interne Funktion
// Daten an Display senden
//--------------------------------------------------------------
void P_N95_DATA(uint8_t wert)
{
  LCD_RAM = wert;
}


//--------------------------------------------------------------
// interne Funktion
// Display ID auslesen (funktioniert nicht immer)
//--------------------------------------------------------------
uint16_t P_N95_ReadID(void)
{
  uint16_t ret_wert=0;
  uint8_t lo,hi;
  
  P_N95_CMD(LCD_RDDID);
  LCD_RAM;      // Dummy
  hi = LCD_RAM; // Manufacture  = 0x29
  lo = LCD_RAM; // Version      = 0x91
  LCD_RAM;      // Dummy

  ret_wert=(hi<<8)|lo; 

  return(ret_wert);  
}


//--------------------------------------------------------------
// kleine Pause (ohne Timer)
//--------------------------------------------------------------
void P_N95_Delay(volatile uint32_t nCount)
{
  while(nCount--)
  {
  }
}
